@@ -21,12 +21,12 @@
  *
  * @author Stephane MANKOWSKI / Guillaume DE BURE
  */
-#include <KUrl>
+#include <qurl.h>
 
-#include <QStringList>
-#include <QHash>
-#include <QDateTime>
-#include <QIcon>
+#include <qstringlist.h>
+#include <qhash.h>
+#include <qdatetime.h>
+#include <qicon.h>
 
 #include "skgerror.h"
 #include "skgdefine.h"
@@ -55,56 +55,33 @@ class SKGDocument;
  */
 typedef QHash<QString, QString> SKGQStringQStringMap;
 
-/**
- * the iterator for SKGQStringQStringMap
- */
-typedef QHash<QString, QString>::Iterator SKGQStringQStringMapIterator;
-
 /**
  * A list of QStringList ==> SKGStringListList
  */
 typedef QList<QStringList> SKGStringListList;
 
-/**
- * the iterator for SKGStringListList
- */
-typedef QList<QStringList>::Iterator SKGStringListListIterator;
-
 /**
  * A list of int ==> SKGIntList
  */
 typedef QList<int> SKGIntList;
 
-/**
- * the iterator for SKGIntList
- */
-typedef QList<int>::Iterator SKGIntListIterator;
-
 /**
 * This class containing various static services
 */
 class SKGBASEMODELER_EXPORT SKGServices : public QObject
 {
     Q_OBJECT
-    /**
-     * This enumerate defines dump modes
-     */
-    Q_ENUMS(DumpMode)
-    /**
-     * This enumerate defines attributes types
-     */
-    Q_ENUMS(AttributeType)
 public:
     /**
      * Unit value
      */
-    typedef struct {
+    struct SKGUnitInfo {
         /** The name of the unit */
         QString Name;
         /** The amount of the unit */
-        double Value;
+        double Value = 0.0;
         /** The number of decimal of the unit*/
-        int NbDecimal;
+        int NbDecimal = 2;
 
         /** The symbol of the unit */
         QString Symbol;
@@ -118,7 +95,9 @@ public:
         QString Parent;
         /** The date of the unit */
         QDate Date;
-    } SKGUnitInfo;
+        /** To know if the unit is obsolete or not */
+        bool Obsolete = false;
+    };
 
     /**
      * This structure defines attributes types
@@ -136,6 +115,11 @@ public:
         OTHER           /**< Other or undefined */
     };
 
+    /**
+     * This enumerate defines attributes types
+     */
+    Q_ENUM(AttributeType)
+
     /**
      * Describe an attribute
      */
@@ -157,14 +141,14 @@ public:
     };
 
     /**
-     * A list of SKGAttributeInfo ==> SKGAttributesList
+     * A list of SKGSearchCriteria ==> SKGAttributesList
      */
-    typedef QList<SKGAttributeInfo> SKGAttributesList;
+    typedef QVector<SKGSearchCriteria> SKGSearchCriteriaList;
 
     /**
-     * the iterator for SKGAttributesList
+     * A list of SKGAttributeInfo ==> SKGAttributesList
      */
-    typedef QList<SKGAttributeInfo>::Iterator SKGAttributesListIterator;
+    typedef QVector<SKGAttributeInfo> SKGAttributesList;
 
     /**
      * This enumerate defines dump modes
@@ -174,21 +158,30 @@ public:
         DUMP_TEXT  /**< To dump in DUMP_TEXT mode */
     };
 
+    /**
+     * This enumerate defines dump modes
+     */
+    Q_ENUM(DumpMode)
+
     /**
      * Get list of search criteria of a string (exxample: +cat1 -abc +auto)
      * @param iString a string to analyze
      * @return the search criteria list
      */
-    static QList<SKGSearchCriteria> stringToSearchCriterias(const QString& iString);
+    static SKGServices::SKGSearchCriteriaList stringToSearchCriterias(const QString& iString);
 
     /**
      * Compute a SQL where clause with criterias and list of attributes
      * @param iSearchCriterias the list of criterias
      * @param iAttributes the list of attributes
      * @param iDocument if a document is given, this document will be used to find the synonyme of attributes
+     * @param iForDisplay to build the human readable string explaining the query
      * @return the where clause
      */
-    static QString searchCriteriasToWhereClause(const QList<SKGSearchCriteria>& iSearchCriterias, const QStringList& iAttributes, const SKGDocument* iDocument = NULL);
+    static QString searchCriteriasToWhereClause(const SKGServices::SKGSearchCriteriaList& iSearchCriterias,
+            const QStringList& iAttributes,
+            const SKGDocument* iDocument,
+            bool iForDisplay = false);
 
     /**
      * Get environment variable
@@ -224,7 +217,7 @@ public:
      * @param iSeparator the separator
      * @return the converted csv string
      */
-    static QString stringsToCsv(const QStringList& iList, const QChar& iSeparator = ';');
+    static QString stringsToCsv(const QStringList& iList, const QChar iSeparator = QLatin1Char(';'));
 
     /**
      * Convert a QString to a html QString
@@ -274,7 +267,8 @@ public:
      * Y for year, example: 2013
      * @return the period string
      */
-    static QString dateToPeriod(const QDate& iDate, const QString& iPeriod);
+    // cppcheck-suppress passedByValue
+    static QString dateToPeriod(const QDate iDate, const QString& iPeriod);
 
     /**
      * Compute the number of working days between two dates.
@@ -282,15 +276,24 @@ public:
      * @param iTo seconde date
      * @return the number of working days.
      */
-    static int nbWorkingDays(const QDate& iFrom, const QDate& iTo);
+    // cppcheck-suppress passedByValue
+    static int nbWorkingDays(const QDate iFrom, const QDate iTo);
 
     /**
      * Convert a QString to QDateTime.
      * @param iDateString the time. The format must be "%Y-%m-%d %H-%M-%S" or "%Y-%m-%d"
-     * @return the converted time_t
+     * @return the converted QDateTime
      */
     static QDateTime stringToTime(const QString& iDateString);
 
+    /**
+     * Convert a QString to QDate.
+     * @param iDateString the date string. Could be a partial date like 1/12.
+     * @param iFixupBackward the date string is fixed in backward mode.
+     * @return the converted QDate
+     */
+    static QDate partialStringToDate(const QString& iDateString, bool iFixupBackward = true);
+
     /**
      * Convert a QDateTime into a QString ready for sql order.
      * The output format is "%Y-%m-%d"
@@ -300,12 +303,13 @@ public:
     static QString dateToSqlString(const QDateTime& iDateTime);
 
     /**
-     * Convert a QDateTime into a QString ready for sql order.
+     * Convert a QDate into a QString ready for sql order.
      * The output format is "%Y-%m-%d"
-     * @param iDateTime the time
+     * @param iDate the date
      * @return the converted QString
      */
-    static QString dateToSqlString(const QDate& iDateTime);
+    // cppcheck-suppress passedByValue
+    static QString dateToSqlString(const QDate iDate);
 
     /**
      * Format a date.
@@ -322,12 +326,12 @@ public:
     static QString dateToSqlString(const QString& iDate, const QString& iFormat);
 
     /**
-     * Get the sql where clause corresponding to a a period date.
-     * @param iPeriod the period date (some thing like 2014 or 2014-04 or 2014-Q2 or 2014-S2
+     * Get the sql where clause corresponding to a period date.
+     * @param iPeriod the period date (some thing like "ALL" "2014" or "2014-04" or "2014-Q2" or "2014-S2"
      * @param iDateAttribute the date attribute
      * @return the where clause
      */
-    static QString getPeriodWhereClause(const QString& iPeriod, const QString& iDateAttribute = "d_date");
+    static QString getPeriodWhereClause(const QString& iPeriod, const QString& iDateAttribute = QStringLiteral("d_date"));
 
     /**
      * Get the last date of a period.
@@ -337,11 +341,12 @@ public:
     static QDate periodToDate(const QString& iPeriod);
 
     /**
-     * Get the previous period.
-     * @param iPeriod the period date (some thing like 2014 or 2014-04 or 2014-Q2 or 2014-S2
-     * @return the previous period
+     * Get the neighboring period.
+     * @param iPeriod the period date (some thing like "ALL" "2014" or "2014-04" or "2014-Q2" or "2014-S2"
+     * @param iDelta the delta to apply (-1 = previous, +1 = next)
+     * @return the neighboring period
      */
-    static QString getPerviousPeriod(const QString& iPeriod);
+    static QString getNeighboringPeriod(const QString& iPeriod, int iDelta = -1);
 
     /**
      * Get the format of a list of string dates
@@ -350,6 +355,23 @@ public:
      */
     static QString getDateFormat(const QStringList& iDates);
 
+    /**
+     * Get the string representing the percentage
+     * @param iAmount the amount
+     * @param iNbDecimal the number of decimals
+     * @return the string.
+     */
+    static QString toPercentageString(double iAmount, int iNbDecimal = 2);
+
+    /**
+     * Get the string representing the amount
+     * @param iAmount the amount
+     * @param iSymbol the symbol
+     * @param iNbDecimal the number of decimals
+     * @return the string.
+     */
+    static QString toCurrencyString(double iAmount, const QString& iSymbol = QString(), int iNbDecimal = 2);
+
     /**
      * Return string ready for sql order
      * @param iString the string
@@ -362,10 +384,19 @@ public:
      * @param iString the csv line
      * @param iSeparator the csv separator
      * @param iCoteDefineBlock to indicate if the character double cote define a block
-     * @param oRealSeparator the real csv separator. If the csv string is strictly all enclosed then oRealSeparator will contain the real separator
+     * @param oRealSeparator the real csv separator. If the csv string is strictly all enclosed then oRealSeparator will contain the real separator. Can be nullptr
      * @return the list of all values in this csv line. Return an empty list if some cote are opened and not closed
      */
-    static QStringList splitCSVLine(const QString& iString, const QChar& iSeparator = ';', bool iCoteDefineBlock = true, QChar* oRealSeparator = NULL);
+    static QStringList splitCSVLine(const QString& iString, const QChar iSeparator, bool iCoteDefineBlock, QChar* oRealSeparator);
+
+    /**
+     * Split a csv line
+     * @param iString the csv line
+     * @param iSeparator the csv separator
+     * @param iCoteDefineBlock to indicate if the character double cote define a block
+     * @return the list of all values in this csv line. Return an empty list if some cote are opened and not closed
+     */
+    static QStringList splitCSVLine(const QString& iString, const QChar iSeparator = QLatin1Char(';'), bool iCoteDefineBlock = true);
 
     /**
      * To dump a table
@@ -384,16 +415,17 @@ public:
 
     /**
      * Copy a sqlite database from memory to file or from file to memory.
-     * It's used to do a load or a save
+     * It is used to do a load or a save
      * @param iFileDb the sqlite pointer corresponding to a file database
      * @param iMemoryDb the sqlite pointer corresponding to  memory database
      * @param iFromFileToMemory
      *        true: the copy is done from iFileDb to iMemoryDb (needed of a load)
      *        false: the copy is done from iMemoryDb to iFileDb (needed of a save)
+     * @param iPassword the password
      * @return An object managing the error
      *   @see SKGError
      */
-    static SKGError copySqliteDatabase(QSqlDatabase* iFileDb, QSqlDatabase* iMemoryDb, bool iFromFileToMemory);
+    static SKGError copySqliteDatabase(const QSqlDatabase& iFileDb, const QSqlDatabase& iMemoryDb, bool iFromFileToMemory, const QString& iPassword = QString());
 
     /**
      * Copy a sqlite database into an XML document.
@@ -402,7 +434,7 @@ public:
      * @return An object managing the error
      *   @see SKGError
      */
-    static SKGError copySqliteDatabaseToXml(QSqlDatabase* iDb, QDomDocument& oDocument);
+    static SKGError copySqliteDatabaseToXml(const QSqlDatabase& iDb, QDomDocument& oDocument);
 
     /**
      * Execute a sqlite orders
@@ -411,38 +443,38 @@ public:
      * @return An object managing the error
      *   @see SKGError
      */
-    static SKGError executeSqliteOrders(QSqlDatabase* iDb, const QStringList& iSqlOrders);
+    static SKGError executeSqliteOrders(const QSqlDatabase& iDb, const QStringList& iSqlOrders);
 
     /**
      * Execute a sqlite order
      * @param iDb a database pointer
      * @param iSqlOrder the sql order
      * @param iBind the binded variables and values
-     * @param iLastId to retrieve the id of the last created object
+     * @param iLastId to retrieve the id of the last created object. Can be nullptr
      * @return An object managing the error
      *   @see SKGError
      */
-    static SKGError executeSqliteOrder(QSqlDatabase* iDb, const QString& iSqlOrder, const QMap<QString, QVariant>& iBind, int* iLastId = NULL);
+    static SKGError executeSqliteOrder(const QSqlDatabase& iDb, const QString& iSqlOrder, const QMap<QString, QVariant>& iBind, int* iLastId);
 
     /**
      * Execute a sqlite order
      * @param iDb a database pointer
      * @param iSqlOrder the sql order
-     * @param iLastId to retrieve the id of the last created object
+     * @param iLastId to retrieve the id of the last created object. Can be nullptr
      * @return An object managing the error
      *   @see SKGError
      */
-    static SKGError executeSqliteOrder(QSqlDatabase* iDb, const QString& iSqlOrder, int* iLastId = NULL);
+    static SKGError executeSqliteOrder(const QSqlDatabase& iDb, const QString& iSqlOrder, int* iLastId = nullptr);
 
     /**
     * Execute a select sqlite order and return the result in @p oResult
     * @param iDb A database pointer
     * @param iSqlOrder the sql order
-    * @param oResult the result of the select. It's a vector of vector of QString
+    * @param oResult the result of the select. It is a vector of vector of QString
     * @return An object managing the error
     *   @see SKGError
     */
-    static SKGError executeSelectSqliteOrder(QSqlDatabase* iDb, const QString& iSqlOrder, SKGStringListList& oResult);
+    static SKGError executeSelectSqliteOrder(const QSqlDatabase& iDb, const QString& iSqlOrder, SKGStringListList& oResult);
 
     /**
      * Execute a select sqlite order returning one value and return the result in @p oResult
@@ -452,18 +484,28 @@ public:
      * @return An object managing the error
      *   @see SKGError
      */
-    static SKGError executeSingleSelectSqliteOrder(QSqlDatabase* iDb, const QString& iSqlOrder, QString& oResult);
+    static SKGError executeSingleSelectSqliteOrder(const QSqlDatabase& iDb, const QString& iSqlOrder, QString& oResult);
 
     /**
     * dump a select sqlite order
     * @param iDb A database pointer
     * @param iSqlOrder the sql order
-    * @param oStream the output stream, NULL for std output (cout)
     * @param iMode dump mode
     * @return An object managing the error
     *   @see SKGError
     */
-    static SKGError dumpSelectSqliteOrder(QSqlDatabase* iDb, const QString& iSqlOrder, QTextStream* oStream = NULL, SKGServices::DumpMode iMode = DUMP_TEXT);
+    static SKGError dumpSelectSqliteOrder(const QSqlDatabase& iDb, const QString& iSqlOrder, SKGServices::DumpMode iMode = DUMP_TEXT);
+
+    /**
+    * dump a select sqlite order
+    * @param iDb A database pointer
+    * @param iSqlOrder the sql order
+    * @param oStream the output stream, nullptr for std output (cout)
+    * @param iMode dump mode
+    * @return An object managing the error
+    *   @see SKGError
+    */
+    static SKGError dumpSelectSqliteOrder(const QSqlDatabase& iDb, const QString& iSqlOrder, QTextStream* oStream, SKGServices::DumpMode iMode = DUMP_TEXT);
 
     /**
      * dump a select sqlite order
@@ -474,7 +516,7 @@ public:
      * @return An object managing the error
      *   @see SKGError
      */
-    static SKGError dumpSelectSqliteOrder(QSqlDatabase* iDb, const QString& iSqlOrder, QString& oResult, SKGServices::DumpMode iMode = DUMP_TEXT);
+    static SKGError dumpSelectSqliteOrder(const QSqlDatabase& iDb, const QString& iSqlOrder, QString& oResult, SKGServices::DumpMode iMode = DUMP_TEXT);
 
     /**
      * dump a select sqlite order
@@ -485,7 +527,7 @@ public:
      * @return An object managing the error
      *   @see SKGError
      */
-    static SKGError dumpSelectSqliteOrder(QSqlDatabase* iDb, const QString& iSqlOrder, QStringList& oResult, SKGServices::DumpMode iMode = DUMP_TEXT);
+    static SKGError dumpSelectSqliteOrder(const QSqlDatabase& iDb, const QString& iSqlOrder, QStringList& oResult, SKGServices::DumpMode iMode = DUMP_TEXT);
 
     /**
      * Read a property file
@@ -509,11 +551,16 @@ public:
      * @param iPassword password
      * @param iEncrypt true to encrypt, false to decrypt
      * @param iHeaderFile the header file (useful for a magic mime type)
+     * @param oModeSQLCipher to know if the file is in SQLite or SQLCipher mode
      * @return An object managing the error
      *   @see SKGError
      */
-    static SKGError cryptFile(const QString& iFileSource, const QString& iFileTarget,
-                              const QString& iPassword, bool iEncrypt = true, const QString& iHeaderFile = "");
+    static SKGError cryptFile(const QString& iFileSource,
+                              const QString& iFileTarget,
+                              const QString& iPassword,
+                              bool iEncrypt,
+                              const QString& iHeaderFile,
+                              bool& oModeSQLCipher);
 
     /**
      * Download a non local URL to a string
@@ -522,7 +569,7 @@ public:
      * @return An object managing the error
      *   @see SKGError
      */
-    static SKGError downloadToStream(const KUrl& iSourceUrl, QByteArray& oStream);
+    static SKGError downloadToStream(const QUrl& iSourceUrl, QByteArray& oStream);
 
     /**
      * Download a non local URL to a temporary file
@@ -531,7 +578,7 @@ public:
      * @return An object managing the error
      *   @see SKGError
      */
-    static SKGError download(const KUrl& iSourceUrl, QString& oTemporaryFile);
+    static SKGError download(const QUrl& iSourceUrl, QString& oTemporaryFile);
 
     /**
      * Upload from a source to a destination
@@ -540,7 +587,7 @@ public:
      * @return An object managing the error
      *   @see SKGError
      */
-    static SKGError upload(const KUrl& iSourceUrl, const KUrl& iDescUrl);
+    static SKGError upload(const QUrl& iSourceUrl, const QUrl& iDescUrl);
 
     /**
      * To enable, disable sql traces
@@ -578,6 +625,22 @@ public:
      */
     static QString encodeForUrl(const QString& iString);
 
+    /**
+     * Return the icon form theme with expected overlays
+     *
+     * @param iName The name of the icon
+     * @param iOverlays List of overlays
+     * @return The icon
+     */
+    static QIcon fromTheme(const QString& iName, const QStringList& iOverlays = QStringList());
+
+    /**
+     * Get a major version (ex: 4.3) of a version (ex: 4.3.1)
+     * @param iVersion the version
+     * @return the major version
+     */
+    static QString getMajorVersion(const QString& iVersion);
+
 private:
     Q_DISABLE_COPY(SKGServices)
     static SKGError m_lastCallbackError;
